﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GeometryFriendsAgents.WorldObjects {
    class Platform : WorldObject, IComparable {

        private const int TOBEINITIALIZED = -1;
        private int _id;

        public int ID {
            get { return _id; }
            set { _id = value; }
        }

        private float _width;

        public float Width {
            get { return _width; }
            set { _width = value; }
        }

        private float _height;

        public float Height {
            get { return _height; }
            set { _height = value; }
        }

        public float Left {
            get { return xPos - Width / 2; }
        }

        public float Right {
            get { return xPos + Width / 2; }
        }

        public float Top { 
            get { return yPos - Height / 2; } 
        }

        private List<Collectible> _collectibles;

        public List<Collectible> Collectibles {
            get { return _collectibles; }
        }

        private int _totalNumberCollectibles;

        public int TotalNumberCollectibles {
            get { return _totalNumberCollectibles; }
        }

        private DateTime _startTime;

        public DateTime StartTime {
            get { return _startTime; }
            set { _startTime = value; }
        }

        private DateTime _endTime;

        public DateTime EndTime {
            get { return _endTime; }
            set { _endTime = value; }
        }

        public Platform(float xPos, float yPos, float height, float width) {
            _xPos = xPos;
            _yPos = yPos;
            _width = width;
            _height = height;
            _totalNumberCollectibles = TOBEINITIALIZED;
            _collectibles = new List<Collectible>();
        }

        public void AddCollectibles(List<Collectible> c) {
            if (_totalNumberCollectibles == -1) {
                _totalNumberCollectibles = c.Count;
            }
            _collectibles = c;
        }

        public int CompareTo(object obj) {
            Platform p = (Platform)obj;

            float myTop = _yPos - _height / 2;
            float hisTop = p.yPos - p.Height / 2;

            float myLeft = _xPos - _width / 2;
            float hisLeft = p.xPos - p.Width / 2;

            if (myTop < hisTop)
                return -1;
            else if (myTop > hisTop)
                return 1;
            else if (myLeft < hisLeft)
                return -1;
            else if (hisLeft < myLeft)
                return 1;
            else
                return 0;
        }

        public int NumberCollectiblesCaught { get { return _totalNumberCollectibles - _collectibles.Count; } }

        public int PercentageCollectiblesCaught {
            get { return (int)(((float)NumberCollectiblesCaught / TotalNumberCollectibles) * 100); }
        }

        internal Platform Clone() {
            Platform p = new Platform(xPos, yPos, Height, Width);
            p._totalNumberCollectibles = _totalNumberCollectibles;
            p._collectibles = new List<Collectible>(_collectibles);
            p.ID = _id;
            return p;
        }
    }
}
